import { Polygon } from "../polygon";
import { Vector } from "../vector";
export declare enum Orientation {
    CCW = 1,
    CW = -1,
    UNORDERED = 0
}
export declare enum UnionStatus {
    isDead = 1,
    isActive = -1,
    isAlive = 0
}
/**
 * Polygon used in poly partition.
 * 1. It is a polygon.
 * 2. Can be either a normal polygon or a hole in outer polygon.
 * 3. Points are sorted in counter-clockwise order from it's center when it's a normal polygon, or clockwise if it's a hole.
 */
export declare class PartitionPolygon extends Polygon {
    isHole: boolean;
    constructor(pos: Vector, points: Array<Vector>, isHole?: boolean);
    getOrientation(): Orientation;
    setOrientation(orientation: Orientation.CCW | Orientation.CW): void;
    invert(): void;
    isValid(): boolean;
}
export declare class UnionPolygon extends Polygon {
    unionStatus: UnionStatus;
    startingVertex: PolyVertex | null;
    vertices: Array<PolyVertex>;
    constructor(pos: Vector, points: Array<Vector>);
}
export declare class PolyVertex extends Vector {
    segmentLength: number;
    intersectingVertex: Array<PolyVertex>;
    intersections: Array<Vector>;
    parent: UnionPolygon | null;
    next: PolyVertex | null;
    normal: Vector | null;
    indexInPolygon: number;
    constructor(x?: number, y?: number);
}
export declare function preprocessIntersection(polys: Array<UnionPolygon>): void;
export declare function unionPolygons(polys: Array<UnionPolygon>): Array<Array<Vector>>;
export declare class PartitionVertex {
    isActive: boolean;
    isConvex: boolean;
    isEar: boolean;
    p: Vector;
    angle: number;
    prev: PartitionVertex | null;
    next: PartitionVertex | null;
    constructor();
}
export declare function inCone(p1: Vector, p2: Vector, p3: Vector, p: Vector): boolean;
export declare function intersects(p11: Vector, p12: Vector, p21: Vector, p22: Vector): boolean;
/**
 * Remove holes from a list of PartitionPolygon.
 * Creates a diagonal from the rightmost hole vertex to some visible vertex.
 * Time complexity: O(h*(n^2)), h is the number of holes, n is the number of vertices.
 * space complexity: O(n)
 * @param {Array} inpolys a list of polygons that can contain holes
 *  vertices of all non-hole polys have to be in counter-clockwise order
 *  vertices of all hole polys have to be in clockwise order
 * @return {boolean} if removing is successful.
 */
export declare function removeHoles(polys: Array<PartitionPolygon>): boolean;
/**
 * Triangulates a polygon by ear clipping
 * Time complexity: O(n^2), n is the number of vertices
 * space complexity: O(n)
 * @param {PartitionPolygon} poly an input polygon to be triangulated
 *  vertices have to be in counter-clockwise order
 * @return {Array} an array of triangles or empty array when failed
 */
export declare function triangulateEC(poly: PartitionPolygon): Array<PartitionPolygon>;
/**
 * Partitions a polygon into convex polygons by using Hertel-Mehlhorn algorithm
 * The algorithm gives at most four times the number of parts as the optimal algorithm.
 * However, in practice it works much better than that and often gives optimal partition.
 * Uses triangulation obtained by ear clipping as intermediate result
 * Time complexity O(n^2), n is the number of vertices
 * space complexity: O(n)
 * @param {PartitionPolygon} poly an input polygon to be partitioned
 *  vertices have to be in counter-clockwise order
 * @return {Array} an array convex polygons or empty array when failed
 */
export declare function convexPartitionHM(poly: PartitionPolygon): Array<PartitionPolygon>;
/**
 * Partitions a polygon into convex polygons by using Hertel-Mehlhorn algorithm
 * The algorithm gives at most four times the number of parts as the optimal algorithm.
 * However, in practice it works much better than that and often gives optimal partition.
 * Uses triangulation obtained by ear clipping as intermediate result
 * Time complexity O(n^2), n is the number of vertices
 * space complexity: O(n)
 * @param {PartitionPolygon} inpolys a list of polygons to be partitioned
 *  vertices of all non-hole polys have to be in counter-clockwise order
 *  vertices of all hole polys have to be in clockwise order
 * @return {Array} an array convex polygons or empty array when failed
 */
export declare function convexPartitionHMList(inpolys: Array<PartitionPolygon>): Array<PartitionPolygon>;
/**
 * Triangulates a list of polygons that may contain holes by ear clipping algorithm
 * first calls RemoveHoles to get rid of the holes, and then Triangulate_EC for each resulting polygon
 * Time complexity: O(h*(n^2)), h is the number of holes, n is the number of vertices
 * Space complexity: O(n)
 * @param {PartitionPolygon} inpolys a list of polygons to be partitioned
 *  vertices of all non-hole polys have to be in counter-clockwise order
 *  vertices of all hole polys have to be in clockwise order
 * @return {Array} an array triangles or empty array when failed
 */
export declare function triangulateECList(inpolys: Array<PartitionPolygon>): Array<PartitionPolygon>;
/**
 * Offset Polygon by offsetting edges without rounding. Assuming old_points is counter-clockwise. Shrink by 'offset' if otherwise.
 * @param {Array} old_points a list of points
 * @param {number} offset distance to edges
 * @return {Array} an array of points of resulting Polygon.
 */
export declare function getEnlargedPolygon(old_points: Array<Vector>, offset: number): Array<Vector>;
