/**
 * The workhorse that represents a navigation mesh built from a series of polygons. Once built, the
 * mesh can be asked for a path from one point to another point. Some internal terminology usage:
 * - neighbor: a polygon that shares part of an edge with another polygon
 * - portal: when two neighbor's have edges that overlap, the portal is the overlapping line segment
 * - channel: the path of polygons from starting point to end point
 * - pull the string: run the funnel algorithm on the channel so that the path hugs the edges of the
 *   channel. Equivalent to having a string snaking through a hallway and then pulling it taut.
 *
 * @class NavMesh
 */
import { Vector } from "../vector";
import { NavPoly } from "./navpoly";
import { Segment } from "../segment";
export declare class NavMesh {
    private _meshShrinkAmount;
    private _navPolygons;
    private _graph;
    private _qtree;
    /**
     * Creates an instance of NavMesh.
     * @param {object[][]} meshPolygonPoints Array where each element is an array of point-like
     * objects that defines a polygon.
     * @param {number} [meshShrinkAmount=0] The amount (in pixels) that the navmesh has been
     * shrunk around obstacles (a.k.a the amount obstacles have been expanded)
     * @memberof NavMesh
     */
    constructor(meshPolygonPoints: Array<Array<Vector>>, meshShrinkAmount?: number);
    /**
     * Get the NavPolys that are in this navmesh.
     *
     * @returns {NavPoly[]}
     * @memberof NavMesh
     */
    getPolygons(): NavPoly[];
    /**
     * Cleanup method to remove references.
     *
     * @memberof NavMesh
     */
    destroy(): void;
    /**
     * Find a path from the start point to the end point using this nav mesh.
     *
     * @param {object} startPoint A point-like object in the form {x, y}
     * @param {object} endPoint A point-like object in the form {x, y}
     * @returns {Vector[]|null} An array of points if a path is found, or null if no path
     *
     * @memberof NavMesh
     */
    findPath(startPoint: Vector, endPoint: Vector): Vector[] | null;
    static areCollinear(edge1: Segment, edge2: Segment): boolean;
    _calculateNeighbors(): void;
    _getSegmentOverlap(line1: Segment, line2: Segment): Vector[] | null;
    /**
     * Project a point onto a polygon in the shortest distance possible.
     *
     * @param {Phaser.Point} point The point to project
     * @param {NavPoly} navPoly The navigation polygon to test against
     * @returns {{point: Phaser.Point, distance: number}}
     *
     * @private
     * @memberof NavMesh
     */
    _projectPointToPolygon(point: Vector, navPoly: NavPoly): {
        point: Vector | null;
        distance: number;
    };
    _projectPointToEdge(point: Vector, line: Segment): Vector;
    private vecTemp;
    getReachableNearTarget(target: Vector, width: number, height?: number): Vector | null;
}
