/**
 * A class that represents a navigable polygon with a navmesh. It is built on top of a
 * {@link Polygon}. It implements the properties and fields that javascript-astar needs - weight,
 * toString, isWall and getCost. See GPS test from astar repo for structure:
 * https://github.com/bgrins/javascript-astar/blob/master/test/tests.js
 *
 * @class NavPoly
 */
import { Polygon } from "../polygon";
import { Vector } from "../vector";
import { AStarNode, GraphNode } from "./navgraph";
import { Segment } from "../segment";
export declare class NavPoly implements GraphNode {
    id: number;
    private polygon;
    boundingRadius: number;
    centroid: Vector;
    private connected;
    portals: Segment[];
    weight: number;
    astar: AStarNode;
    edges: Segment[];
    /**
     * Creates an instance of NavPoly.
     * @param {number} id
     * @param {Polygon} polygon
     *
     * @memberof NavPoly
     */
    constructor(id: number, polygon: Polygon);
    /**
     * Returns an array of points that form the polygon.
     *
     * @returns {Vector[]}
     * @memberof NavPoly
     */
    getPoints(): Vector[];
    /**
     * Check if the given point-like object is within the polygon
     *
     * @param {object} point Object of the form {x, y}
     * @returns {boolean}
     * @memberof NavPoly
     */
    contains(point: Vector): boolean;
    /**
     * Calculate the radius of a circle that circumscribes the polygon.
     *
     * @returns {number}
     * @memberof NavPoly
     */
    calculateRadius(): number;
    /**
     * Check if the given point-like object is on one of the edges of the polygon.
     *
     * @param {object} Point-like object in the form { x, y }
     * @returns {boolean}
     * @memberof NavPoly
     */
    isPointOnEdge(p: Vector): boolean;
    destroy(): void;
    toString(): string;
    isWall(): boolean;
    centroidDistance(navPolygon: NavPoly): number;
    getCost(navPolygon: GraphNode): number;
    neighbors(): Array<GraphNode>;
}
